

{-----------------------------------------------------------------------------
 Unit Name: DrawImplicitSurf
 Project Name : Superficies
 Author:    Angel Montesinos Amilibia
 05/01/02 
 Purpose: Implementation of a ray tracing for drawing a surface given  in
        implicit form as f(x,y,z)= 0.
-----------------------------------------------------------------------------}
{*-
 * Copyright (c) 2002 Angel Montesinos Amilibia
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *        Version 4.0.0.0
 *}



unit DrawImplicitSurf;
interface
uses
  Windows,
{Types, QGraphics}
  Globals;



function DrawImplicitSurface: Boolean;
function ReflectedLight(ptSurface: TPoint; var anver: integer; var light:
  double): Boolean;
function ComputeNormal(thePos: Vector3): Vector3;
function PickedPtImplicit(theMouse: TPoint; var pos: Vector3): Boolean;
procedure ComputeNormalPairImplicit;


implementation

uses
  SysUtils, Math, Forms, mmSystem,
{DateUtils, mmSystem, QForms}
  FSurface, OCFunct, Utilities, InitParams, ManageClips, Antialiasing;

function ComputeNormal(thePos: Vector3): Vector3;
var
  partials, values, here: Vector3;
  f00, l2: double;
  i: integer;
begin
  f00 := gTheF1.V(thePos);
  here := thePos;
  here[1] := thePos[1] + gIncr3D[1];
  values[1] := gTheF1.V(here);
  here[1] := thePos[1];
  here[2] := thePos[2] + gIncr3D[2];
  values[2] := gTheF1.V(here);
  here[2] := thePos[2];
  here[3] := thePos[3] + gIncr3D[3];
  values[3] := gTheF1.V(here);
  for i := 1 to 3 do
    partials[i] := (values[i] - f00) / gIncr3D[i];
  l2 := 1 / EuclNorm3(partials);
  for i := 1 to 3 do
    ComputeNormal[i] := l2 * partials[i];
end; {ComputeNormal}

function PickedPtImplicit(theMouse: TPoint; var pos: Vector3): Boolean;
var
  a, b, root: double;
  ray: Vector3;
  k: Integer;
begin
  Result := False;
  ray := FindRay(theMouse);
  SegmentInCube(gViewpoint, ray, gVpCubeKey, a, b);
  if a < b then begin
    if FirstZero(gViewpoint, ray, a, b, root) then begin
      for k := 1 to 3 do
        pos[k] := gViewpoint[k] + root * ray[k];
      Result := True;
    end;
  end;
end;




function ReflectedLight(ptSurface: TPoint; var anver: integer; var light:
  double): Boolean;
const
  dullness: double = 0.3;
  OneMinusDullness: double = 1 - 0.3;
  FocusIntens: double = 0.60;
  OneMinusFocusIntens: double = 0.4;

var
  lambda, sigma, omega, compF, compAmb, f2x, m2x: double;
  normal, thePos, toVp, toFocus: Vector3;
  i: Integer;
  visFromFocus, goodPt: Boolean;
begin
  goodPt := PickedPtImplicit(ptSurface, thePos);
  ReflectedLight := goodPt;
  if goodPt then begin
    for i := 1 to 3 do begin
      toVp[i] := gViewpoint[i] - thePos[i];
      toFocus[i] := gFocus[i] - thePos[i];
    end;
    m2x := EuclNorm3(toVp);
    normal := ComputeNormal(thePos);
    sigma := EuclInnerProd3(toVp, Normal) / m2x;
    compAmb := (1 + abs(sigma)) / 2;
    visFromFocus := IsVisibleImplicit(gFocus, thePos, gFCubeKey, TRUE);
    if visFromFocus then begin
      f2x := EuclNorm3(toFocus);
      lambda := EuclInnerProd3(toFocus, normal) / f2x;
      omega := 2 * lambda * sigma;
      visFromFocus := (omega > 0);
    end;
    if visFromFocus then begin
      lambda := abs(lambda);
      omega := (1 + omega - EuclInnerProd3(toFocus, toVp) / (m2x * f2x)) /
        2;
          //omega := sqr(sqr(omega));
      compF := {lambda * dullness + OneMinusDullness * omega;} lambda *
        (OneMinusDullness
        * sqr(omega) + dullness * (1 +
        abs(sigma)) / 2);
    end
    else
      compF := 0;
    light := Min(0.99, FocusIntens * compF + OneMinusFocusIntens * compAmb);
    if (sigma > 0) then
      anver := 0
    else
      anver := 1;
  end; {de IF goodPt}
end;





function ComputeImplicitSurface: Boolean;
const
  maxFocus: double = 5.0;
var
  j, k: integer;
  thePos: TPoint;
  light: double;
  index: byte;
  anver: integer;
  line: pTColorArray;
begin
  result := False;
  for k := 0 to kHeightMinus1 do begin
    if (k mod 3 = 0) then
      Application.ProcessMessages;
    if gStop then
      Exit;
    thePos.y := k;
    line := gStdBmp.ScanLine[k];
    for j := 0 to kWidthMinus1 do begin
      thePos.x := j;
      if ReflectedLight(thePos, anver, light) then begin
        index := round(99.7 * light);
        if (anver = 0) then
          index := 101 + index
        else
          index := abs(index) + 1;
        Assert(index <= 200);
        line^[j] := gPalette3D^[index];
        gMap^[j, k] := index;
      end;
    end;
  end;
  result := True;
end; 



function DrawImplicitSurface: Boolean;
var
{$IFDEF LINUX}
  present, past := TDateTime;
{$ENDIF}
{$IFDEF MSWINDOWS}
  present, past: Integer;
{$ENDIF}
  milliSeconds: Integer;
begin
  FillChar(gMap^, kWidth * kHeight, 0);
{$IFDEF LINUX}
  past := Time;
{$ENDIF}
{$IFDEF MSWINDOWS}
  past := TimeGetTime;
{$ENDIF}
  SetFocus;
  FrameFromFocus;
  ComputeVpCubeKey;
  ComputeFocusCubeKey;
  result := ComputeImplicitSurface;
{$IFDEF LINUX}
  past := Time;
  milliSeconds := MilliSecondsBetween(present, past)
{$ENDIF}
{$IFDEF MSWINDOWS}
  present := TimeGetTime;
  milliSeconds := present - past;
{$ENDIF}
  gGrain := Min(9, trunc(sqrt(milliSeconds / 500)));
  Antialiase(gMap);
  ReceiveColorMap(gStdBmp, gMap);
  surface.Drawing3D.Invalidate;
  ComputeCube;
  PrepareToSetClip;
end;


procedure ComputeNormalPairImplicit;
const
  EySin: double = 1 / 15;
  EyCos: double = 0.9977753031;
var
  nLeft, hLeft, vLeft: Vector3;
  j, k, anver, half, index: integer;
  thePos: TPoint;
  light, d: double;
  prevC0, prevC1, prevC2, prevC3: double;
  prevPcx: Integer;
  cen, prevH, prevV, prevN, prevVp: Vector3;
  line: pTColorArray;
begin
  FillChar(gNormalPairMap^, kWidth * kHeight, 0);
  prevC0 := gC0;
  gC0 := prevC0 / 2;
  prevPcx := gpcx;
  gpcx := prevPcx div 2;
  cen := gCenter3D;
  d := EuclDistance3(gViewpoint, cen);
  vLeft := gV;
  for j := 1 to 3 do begin
    nLeft[j] := EyCos * gN[j] + EySin * gH[j];
    hLeft[j] := -EySin * gN[j] + EyCos * gH[j];
  end;
  gMovingBmp.canvas.FillRect(gRect);
  SetFocus;
  ComputeVPCubeKey;
  ComputeFocusCubeKey;
  half := kWidth div 2;
  for k := 0 to kHeightMinus1 do begin
    if (k mod 3 = 0) then Application.ProcessMessages;
    if gStop then
      Exit;
    thePos.y := k;
    for j := 1 to half do begin
      thePos.x := j;
      if ReflectedLight(thePos, anver, light) then begin
        index := round(99.7 * light);
        if (anver = 0) then
          index := 101 + index
        else
          index := abs(index) + 1;
        gNormalPairMap^[j, k] := index;
      end;
    end;
  end;


  prevC1 := gC1;
  prevC2 := gC2;
  prevC3 := gC3;
  gC3 := EuclInnerProd3(nLeft, cen) - d;
  gC1 := EuclInnerProd3(hLeft, cen);
  gC2 := EuclInnerProd3(vLeft, cen);
  prevVp := gViewPoint;
  for j := 1 to 3 do
    gViewPoint[j] := gC3 * nLeft[j] + gC1 * hLeft[j] + gC2 * vLeft[j];
  gpcx := gpcx + kWidth div 2;
  prevH := gH;
  prevN := gN;
  prevV := gV;
  gH := hLeft;
  gV := vLeft;
  gN := nLeft;

  for k := 1 to kHeight - 1 do begin
    if (k mod 3 = 0) then Application.ProcessMessages;
    if gStop then
      Exit;
    thePos.y := k;
    for j := half + 1 to kWidthMinus1 do begin
      thePos.x := j;
      if ReflectedLight(thePos, anver, light) then begin
        index := round(99.7 * light);
        if (anver = 0) then
          index := 101 + index
        else
          index := abs(index) + 1;
        gNormalPairMap^[j, k] := index;
      end;
    end;
  end;

  gC0 := prevC0;
  gC1 := prevC1;
  gC2 := prevC2;
  gC3 := prevC3;
  gpcx := prevPcx;
  gViewPoint := prevVp;
  gH := prevH;
  gV := prevV;
  gN := prevN;

  Antialiase(gNormalPairMap);
  for k := 0 to kHeightMinus1 do begin
    line := gMovingBmp.ScanLine[k];
    for j := 0 to kWidthMinus1 do
      line^[j] := gPalette3D^[gNormalPairMap^[j, k]];
  end;
end;

end.
