

{-----------------------------------------------------------------------------
 Unit Name: FSelectToPrint
 Project Name : Superficies
 Author:    Angel Montesinos Amilibia
 05/01/02 
 Purpose: by means of TForm2, the user can choose the contents and resolution
    for the printing of a surface
-----------------------------------------------------------------------------}
{*-
 * Copyright (c) 2002 Angel Montesinos Amilibia
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *        Version 4.0.0.0
 *}



unit FSelectToPrint;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ActnList, StdActns, ExtCtrls;

type
  TForm2 = class(TForm)
    CoordinatesCheck: TCheckBox;
    Label1 : TLabel;
    CurvaturesCheck: TCheckBox;
    AsymptoticsCheck: TCheckBox;
    IntrinsicCheck: TCheckBox;
    Label2 : TLabel;
    CancelBt: TButton;
    OKBt: TButton;
    GeodBallsCheck: TCheckBox;
    PrintedPageContents: TRadioGroup;
    LightRaysCheck: TCheckBox;
    ParalTransportCheck: TCheckBox;
    Resolution: TRadioGroup;
    Label3: TLabel;
    procedure OnClose(Sender : TObject; var Action : TCloseAction);
    procedure OnShowForm(Sender : TObject);
    procedure OnOK(Sender : TObject);
    procedure OnCancel(Sender : TObject);
    procedure OnPrintedPageContents(Sender: TObject);

  PRIVATE
    { Private declarations }
  PUBLIC
  { Public declarations }
  end;

var
  Form2 : TForm2;

implementation

{$R *.DFM}
uses
  Globals;



procedure TForm2.OnClose(Sender : TObject; var Action : TCloseAction);
begin
  if (modalResult <> mrOK) then
    ModalResult := mrCancel;
end;

procedure TForm2.OnShowForm(Sender : TObject);
begin
  visible := True;
  if gType = PseudoRiemannian then begin
    PrintedPageContents.Items[0] := 'Print Background only';
    PrintedPageContents.Items[1] := 'Background and Lines';
    PrintedPageContents.Controls[2].Enabled := False;
    CoordinatesCheck.Enabled := False;
    CurvaturesCheck.Enabled := False;
    AsymptoticsCheck.Caption := 'Lines of Const. Curv.';
    IntrinsicCheck.caption := 'Geodesics';
  end
  else begin
    PrintedPageContents.Controls[1].Enabled := (gMode = Standard);
    PrintedPageContents.Controls[2].Enabled := (gMode = Standard) and (gType <> Implicit);
  end;
  OnPrintedPageContents(nil);
  resolution.Enabled := not (gMode in gWireModes);
end;

procedure TForm2.OnOK(Sender : TObject);
var
  count, i, answer : Integer;
begin
  if gType <> PseudoRiemannian then begin
    gLinesData[kCoordinates].toPrint := CoordinatesCheck.checked;
    gLinesData[kCurvature].toPrint := CurvaturesCheck.checked;
    gLinesData[kAsymptotics].toPrint := AsymptoticsCheck.checked;
    gLinesData[kIntrinsic].toPrint := IntrinsicCheck.checked;
    gLinesData[kGeodBall].toPrint:= GeodBallsCheck.checked;
    gLinesData[kLightRays].toPrint:= LightRaysCheck.checked;
    gLinesData[kTransport].toPrint:= ParalTransportCheck.checked;
    gLinesData[kArrows].toPrint:= ParalTransportCheck.checked;
  end
  else begin
    gLinesData[kConstantCurvatureLorentz].toPrint := AsymptoticsCheck.checked;
    gLinesData[kConstantCurvatureRiem].toPrint := AsymptoticsCheck.checked;
    gLinesData[kIntrinsicLorentz].toPrint := IntrinsicCheck.checked;
    gLinesData[kIntrinsicRiem].toPrint := IntrinsicCheck.checked;
    gLinesData[kGeodBall].toPrint:= GeodBallsCheck.checked;
    gLinesData[kLightRays].toPrint:= LightRaysCheck.checked;
    gLinesData[kTransport].toPrint:= ParalTransportCheck.checked;
    gLinesData[kArrows].toPrint:= ParalTransportCheck.checked;
  end;
  count := 0;
  gPrintLines := PrintedPageContents.ItemIndex > 0;
  gPrintSurface := PrintedPageContents.ItemIndex < 2;
  for i := kCoordinates to kTransport do
    count := count + ord(gLinesData[i].toPrint);
  gResolution := resolution.ItemIndex + 1;
  gResolution := 400*gResolution;
  if gPrintSurface or (count > 0) then
    ModalResult := mrOK
  else begin
    answer:= Application.MessageBox('You did select nothing to print!',
      'Warning', MB_OKCANCEL + MB_DEFBUTTON1);
    if answer <> IDCancel then
      ModalResult := mrCancel
    else
      modalResult := 0;
  end;
end;

procedure TForm2.OnCancel(Sender : TObject);
begin
  ModalResult := mrCancel;
end;



procedure TForm2.OnPrintedPageContents(Sender: TObject);
begin
  if PrintedPageContents.ItemIndex = 0 then begin
    CoordinatesCheck.Enabled:= False;
    CurvaturesCheck.Enabled:= False;
    AsymptoticsCheck.Enabled:= False;
    IntrinsicCheck.Enabled:= False;
    GeodBallsCheck.Enabled:= False;
    LightRaysCheck.Enabled:= False;
    ParalTransportCheck.Enabled:= False;
    Label1.Enabled := False;
    Label2.Enabled:= False;
    end
  else begin
    CoordinatesCheck.Enabled:= (gMode = Standard);
  Resolution.Enabled := ( PrintedPageContents.ItemIndex <> 2);


  CurvaturesCheck.Enabled:= (gMode = Standard);
  AsymptoticsCheck.Enabled:= (gMode = Standard);
  IntrinsicCheck.Enabled:= (gMode = Standard);
  GeodBallsCheck.Enabled:= (gMode = Standard);
  LightRaysCheck.Enabled:= (gMode = Standard);
  ParalTransportCheck.Enabled:= (gMode = Standard);
  Label1.Enabled := (gMode = Standard);
  Label2.Enabled:= (gMode = Standard);
  CoordinatesCheck.Enabled := (gMode = Standard) and (gType <> PseudoRiemannian);
  CurvaturesCheck.Enabled := (gMode = Standard) and (gType <> PseudoRiemannian);
  ParalTransportCheck.Checked:= (gMode = Standard) and (gType <> Implicit);
  ParalTransportCheck.Enabled := (gMode = Standard) and (gType <> Implicit);
  end;
end;


end.



